import { getSafeIndex, getStepValue, getTargetIndex } from "./common/utils";
class Scheduler {
    constructor({ dataCount = 0, defaultDataIndex = 0, minCount = 3, loop = false, onDataIndexChange, onRecompute, onRestart, onContainerIndexChange }) {
        this.group = [];
        this.indexMapping = new Map();
        this.containerIndex = 0;
        this.dataCount = dataCount;
        this.dataIndex = defaultDataIndex;
        this.minCount = minCount;
        this.loop = loop;
        this.onDataIndexChange = onDataIndexChange;
        this.onRecompute = onRecompute;
        this.onRestart = onRestart;
        this.onContainerIndexChange = onContainerIndexChange;
    }
    get maxDataIndex() {
        if (this.dataCount) {
            return this.dataCount - 1;
        }
        return this.minCount - 1;
    }
    get groupSize() {
        if (this.loop) {
            return this.minCount;
        }
        if (this.dataCount <= this.minCount)
            return this.dataCount;
        // [开始新增点， 结束新增点]
        const overCount = this.dataCount % this.minCount;
        const maxGroupSize = overCount + this.minCount;
        const maxCount = this.minMiddleCount + 1;
        const endIndex = this.dataCount - maxCount - 1;
        const increaseCount = overCount - 1;
        const startIndex = endIndex - increaseCount;
        if (this.dataIndex <= startIndex)
            return this.minCount;
        if (this.dataIndex > startIndex && this.dataIndex <= endIndex) {
            return this.dataIndex - startIndex + this.minCount;
        }
        return maxGroupSize;
    }
    get maxContainerIndex() {
        if (this.dataCount) {
            return this.groupSize - 1;
        }
        return this.minCount - 1;
    }
    get minMiddleCount() {
        return Math.floor(this.minCount / 2);
    }
    /**
     * 更新数据总量
     * @param count 总量值
     */
    updateCount(count) {
        this.dataCount = count;
    }
    /**
     * 更新当前数据索引
     * @param index 当前索引值
     */
    async updateIndex(index) {
        var _a, _b, _c;
        const preDataIndex = this.dataIndex;
        const getIndex = this.loop ? getTargetIndex : getSafeIndex;
        this.dataIndex = getIndex(index, this.maxDataIndex);
        this.setContainerIndex(this.computeContainerIndex());
        if (preDataIndex !== this.dataIndex) {
            (_a = this.onDataIndexChange) === null || _a === void 0 ? void 0 : _a.call(this, { containerIndex: this.containerIndex, dataIndex: this.dataIndex });
        }
        if (!this.loop && Math.abs(this.dataIndex - preDataIndex) > 1) {
            (_b = this.onRestart) === null || _b === void 0 ? void 0 : _b.call(this, { containerIndex: this.containerIndex, dataIndex: this.dataIndex });
        }
        if (this.loop && Math.abs(getStepValue(preDataIndex, this.dataIndex, this.maxDataIndex)) > 1) {
            (_c = this.onRecompute) === null || _c === void 0 ? void 0 : _c.call(this);
        }
        return this.dataIndex;
    }
    recomputeIndexArr() {
        if (!this.dataCount)
            return [];
        this.setContainerIndex(this.computeContainerIndex());
        this.setGroup(this.computeGroup());
        return this.group;
    }
    computeContainerIndex(step) {
        if (this.loop) {
            return getTargetIndex(this.containerIndex + (step || 0), this.maxContainerIndex);
        }
        if (this.dataCount <= this.minCount * 2 - 1)
            return this.dataIndex;
        const overCount = this.dataCount % this.minCount;
        const lastGroupStartDataIndex = this.dataCount - overCount - this.minCount;
        if (this.dataIndex <= lastGroupStartDataIndex)
            return this.dataIndex % this.minCount;
        return this.dataIndex - lastGroupStartDataIndex;
    }
    setContainerIndex(index) {
        var _a;
        const preContainerIndex = this.containerIndex;
        this.containerIndex = index;
        if (preContainerIndex !== index) {
            (_a = this.onContainerIndexChange) === null || _a === void 0 ? void 0 : _a.call(this, { containerIndex: this.containerIndex, dataIndex: this.dataIndex });
        }
        return this.containerIndex;
    }
    getStepOffset(targetIndex) {
        return getStepValue(this.dataIndex, targetIndex, this.maxDataIndex, this.loop);
    }
    getSafeDataIndex(dataIndex) {
        return getSafeIndex(dataIndex, this.maxDataIndex);
    }
    getDataIndexByStep() {
    }
    getDataIndex() {
        return this.dataIndex;
    }
    getContainerIndex() {
        return this.containerIndex;
    }
    getDataIndexByContainerIndex(index) {
        return this.indexMapping.get(index);
    }
    /**
     * 非 `loop` 模式时， 特殊处理第一组数据
     * @private
     */
    getFirstGroupIndexMapping() {
        return new Map(Array(Math.min(this.minCount, this.dataCount)).fill(null).map((_, v) => [v, v]));
    }
    /**
     * 非 `loop` 模式时， 特殊处理最后一组数据
     * @returns
     */
    getLastGroupIndexMapping() {
        if (this.dataCount <= this.minCount) {
            return new Map(Array(this.dataCount).fill(null).map((_, i) => [i, i]));
        }
        const overCount = this.dataCount % this.minCount;
        const groupSize = this.groupSize;
        const lastGroupStartDataIndex = this.dataCount - overCount - this.minCount;
        const map = new Map();
        for (let i = 0; i < groupSize; i++) {
            const tempMarkIndex = lastGroupStartDataIndex + i;
            map.set(i, tempMarkIndex);
        }
        return map;
    }
    get isFirstGroup() {
        return this.dataIndex <= this.minMiddleCount;
    }
    get isLastGroup() {
        // 开始超出minCount 就是最后一页了
        if (this.dataCount % this.minCount === 0) {
            return this.dataIndex >= this.maxDataIndex - this.minMiddleCount;
        }
        return this.groupSize > this.minCount;
    }
    getIndexMapping() {
        if (!this.loop) {
            if (this.isFirstGroup)
                return this.getFirstGroupIndexMapping();
            if (this.isLastGroup)
                return this.getLastGroupIndexMapping();
        }
        const map = new Map();
        map.set(this.containerIndex, getTargetIndex(this.dataIndex, this.maxDataIndex));
        Array(this.minMiddleCount).fill(null).forEach((_, _index) => {
            const count = _index + 1;
            // 计算前后位置
            const prevSwiperIndex = getTargetIndex(this.containerIndex - count, this.maxContainerIndex);
            const nextSwiperIndex = getTargetIndex(this.containerIndex + count, this.maxContainerIndex);
            const prevMarkIndex = getTargetIndex(this.dataIndex - count, this.maxDataIndex);
            const nextMarkIndex = getTargetIndex(this.dataIndex + count, this.maxDataIndex);
            map.set(prevSwiperIndex, prevMarkIndex);
            map.set(nextSwiperIndex, nextMarkIndex);
        });
        return map;
    }
    computeGroup() {
        this.indexMapping = this.getIndexMapping();
        const arr = [];
        for (const [key, value] of this.indexMapping) {
            arr[key] = value;
        }
        return arr;
    }
    setGroup(group) {
        this.group = group;
    }
}
export default Scheduler;
//# sourceMappingURL=Scheduler.js.map